==> storm-sources-0.002.tar.gz: SToRM/filtering.h <==
#ifndef FILTERING_H_
#define FILTERING_H_

#include "index.h"
#include "alignment.h"

typedef char REGION_FILTER_MASK_TYPE;

#define REGION_MASK_AVAILABLE 0x0
#define REGION_MASK_ALIGNED   0x1

/* A region filter is an array with the same size as the reference, where region
 * s already aligned are masked, so that hits inside them are prevented from being
 * extended again */

typedef struct RegionFilterType {
  int size;
  int window_size;
  REGION_FILTER_MASK_TYPE* mask;
  int* mask_positions;
  int masked_counter;
} RegionFilterType;

/**
 * Create a filter, for a given reference database, that is based on a given index.
 * @param db
 * @param index
 * @return the address of the created filter
 */
RegionFilterType* reg_filter__create(ReferenceDBType* db, int window_size);

/**
 * Initialize a filter, for a given reference database, that is based on a given index.
 * @param filter
 * @param db
 * @param index
 */
inline void reg_filter__init(RegionFilterType* filter, ReferenceDBType* db, const int window_size);

/**
 * Resets a filter, by erasing the masked zones (nothing is aligned), resetting the crt_hit field,
 * and setting a new index if the index parameter is not null
 * @param filter The filter to reset
 * @param index The new index. If this parameter is NULL, the previous index is kept.
 */
inline void reg_filter__reset(RegionFilterType* filter) ;


/**
 * Finds the next hit of a key in the index. If the next hit is in an already aligned zone,
 * the search continues until the hit is in an available zone or there are no more hits
 * for this key in the index.
 * @param filter
 * @param key
 * @return The next hit that is not in an already aligned zone, or -1 if there is no such hit.
 */
inline int reg_filter__next_hit(RegionFilterType* filter, IndexType* index, const int key, const int _ogtn);

/**
 * Finds the next hit of a key in the index. If the next hit is in an already aligned zone,
 * the search continues until the hit is in an available zone or there are no more hits
 * for this key in the index.
 * @param filter
 * @param key
 * @return The next hit that is not in an already aligned zone, or -1 if there is no such hit.
 */
inline int reg_filter__extern_next_hit(RegionFilterType* filter, IndexType* index, const int key, int* hit_pointer);


/**
 * Masks the part of the reference that was successfully aligned, so that the next
 * hit search can avoid it.
 * @param filter
 * @param alignment The alignment of the reference part
 * @param reference_start The position of the aligned part in the reference
 * @return 0 if the masking was successful, -1 if the alignment data is wrong
 * (alignment not performed yet in the given structure).
 */
inline int reg_filter__mask(RegionFilterType* filter, AlignmentType* alignment, const int reference_start);
/**
 * Finds the next <tt>count</tt> hits of a key in the index, avoiding already matched zones.
 * This is useful for allowing the quick alignment filter perform several alignments simultaneously.
 * @param filter
 * @param key
 * @param dest a vector where the hit positions are placed
 * @param count the humber of hits to find
 * @return The last hit placed in dest, or -1 if there is no such hit.
 */
inline int reg_filter__next_hits(RegionFilterType* filter, IndexType* index, const int key, int* dest, const int count, const int _ogtn);
/**
 * Finds the next <tt>count</tt> hits of a key in the index, avoiding already matched zones.
 * This is useful for allowing the quick alignment filter perform several alignments simultaneously.
 * @param filter
 * @param key
 * @param dest a vector where the hit positions are placed
 * @param count the humber of hits to find
 * @return The last hit placed in dest, or -1 if there is no such hit.
 */
inline int reg_filter__extern_next_hits(RegionFilterType* filter, IndexType* index, const int key, int* dest, const int count, int* hit_pointer);

/**
 * Cleans up the memory allocated by a filter
 * @param filter
 */
inline void reg_filter__destroy(RegionFilterType* filter);

#endif /* FILTERING_H_ */

==> storm-sources-0.004.tar.gz: SToRM/filtering.h <==
#ifndef FILTERING_H_
#define FILTERING_H_

#include "index.h"
#include "alignment.h"

typedef char REGION_FILTER_MASK_TYPE;

#define REGION_MASK_AVAILABLE 0x0
#define REGION_MASK_ALIGNED   0x1

/* A region filter is an array with the same size as the reference, where region
 * s already aligned are masked, so that hits inside them are prevented from being
 * extended again */

typedef struct RegionFilterType {
  int size;
  int window_size;
  REGION_FILTER_MASK_TYPE* mask;
  int* mask_positions;
  int masked_counter;
} RegionFilterType;

/**
 * Create a filter, for a given reference database, that is based on a given index.
 * @param db
 * @param index
 * @return the address of the created filter
 */
RegionFilterType* reg_filter__create(ReferenceDBType* db, const int window_size);

/**
 * Initialize a filter, for a given reference database, that is based on a given index.
 * @param filter
 * @param db
 * @param index
 */
inline void reg_filter__init(RegionFilterType* filter, ReferenceDBType* db, const int window_size);

/**
 * Resets a filter, by erasing the masked zones (nothing is aligned), resetting the crt_hit field,
 * and setting a new index if the index parameter is not null
 * @param filter The filter to reset
 * @param index The new index. If this parameter is NULL, the previous index is kept.
 */
inline void reg_filter__reset(RegionFilterType* filter) ;


/**
 * Finds the next hit of a key in the index. If the next hit is in an already aligned zone,
 * the search continues until the hit is in an available zone or there are no more hits
 * for this key in the index.
 * @param filter
 * @param key
 * @return The next hit that is not in an already aligned zone, or -1 if there is no such hit.
 */
inline int reg_filter__next_hit(RegionFilterType* filter, IndexType* index, const int key, const int _ogtn);

/**
 * Finds the next hit of a key in the index. If the next hit is in an already aligned zone,
 * the search continues until the hit is in an available zone or there are no more hits
 * for this key in the index.
 * @param filter
 * @param key
 * @return The next hit that is not in an already aligned zone, or -1 if there is no such hit.
 */
inline int reg_filter__extern_next_hit(RegionFilterType* filter, IndexType* index, const int key, int* hit_pointer);


/**
 * Masks the part of the reference that was successfully aligned, so that the next
 * hit search can avoid it.
 * @param filter
 * @param alignment The alignment of the reference part
 * @param reference_start The position of the aligned part in the reference
 * @return 0 if the masking was successful, -1 if the alignment data is wrong
 * (alignment not performed yet in the given structure).
 */
inline int reg_filter__mask(RegionFilterType* filter, AlignmentType* alignment, const int reference_start);
/**
 * Finds the next <tt>count</tt> hits of a key in the index, avoiding already matched zones.
 * This is useful for allowing the quick alignment filter perform several alignments simultaneously.
 * @param filter
 * @param key
 * @param dest a vector where the hit positions are placed
 * @param count the humber of hits to find
 * @return The last hit placed in dest, or -1 if there is no such hit.
 */
inline int reg_filter__next_hits(RegionFilterType* filter, IndexType* index, const int key, int* dest, const int count, const int _ogtn);
/**
 * Finds the next <tt>count</tt> hits of a key in the index, avoiding already matched zones.
 * This is useful for allowing the quick alignment filter perform several alignments simultaneously.
 * @param filter
 * @param key
 * @param dest a vector where the hit positions are placed
 * @param count the humber of hits to find
 * @return The last hit placed in dest, or -1 if there is no such hit.
 */
inline int reg_filter__extern_next_hits(RegionFilterType* filter, IndexType* index, const int key, int* dest, const int count, int* hit_pointer);

/**
 * Cleans up the memory allocated by a filter
 * @param filter
 */
inline void reg_filter__destroy(RegionFilterType* filter);

#endif /* FILTERING_H_ */
